@@ -3,7 +3,7 @@
 #
 #  (C) Paul Evans, 2021 -- leonerd@leonerd.org.uk
 
-package XS::Parse::Keyword::Builder 0.06;
+package XS::Parse::Keyword::Builder 0.08;
 
 use v5.14;
 use warnings;
@@ -140,32 +140,34 @@ enum {
 
 enum {
   /* skip zero */
-  XS_PARSE_KEYWORD_LITERALCHAR = 1,
-  XS_PARSE_KEYWORD_LITERALSTR,
-  XS_PARSE_KEYWORD_FAILURE = 0x0f,
 
-  XS_PARSE_KEYWORD_BLOCK = 0x10,  /* op */
-  XS_PARSE_KEYWORD_ANONSUB,       /* cv */
+  /*                                    emits */
+  XS_PARSE_KEYWORD_LITERALCHAR = 1,   /* nothing */
+  XS_PARSE_KEYWORD_LITERALSTR,        /* nothing */
+  XS_PARSE_KEYWORD_FAILURE = 0x0f,    /* nothing */
+
+  XS_PARSE_KEYWORD_BLOCK = 0x10,      /* op */
+  XS_PARSE_KEYWORD_ANONSUB,           /* cv */
   /* TODO: XS_PARSE_KEYWORD_ARITHEXPR = 0x12 */
-  XS_PARSE_KEYWORD_TERMEXPR = 0x13, /* op */
-  XS_PARSE_KEYWORD_LISTEXPR,      /* op */
+  XS_PARSE_KEYWORD_TERMEXPR = 0x13,   /* op */
+  XS_PARSE_KEYWORD_LISTEXPR,          /* op */
   /* TODO: XS_PARSE_KEYWORD_FULLEXPR = 0x15 */
-  XS_PARSE_KEYWORD_IDENT = 0x16,  /* sv */
-  XS_PARSE_KEYWORD_PACKAGENAME,   /* sv */
-  XS_PARSE_KEYWORD_LEXVARNAME,    /* sv */
-  XS_PARSE_KEYWORD_LEXVAR,        /* padix */
-  XS_PARSE_KEYWORD_ATTRS,         /* attr.name + attr.val */
-  XS_PARSE_KEYWORD_VSTRING,       /* sv */
+  XS_PARSE_KEYWORD_IDENT = 0x16,      /* sv */
+  XS_PARSE_KEYWORD_PACKAGENAME,       /* sv */
+  XS_PARSE_KEYWORD_LEXVARNAME,        /* sv */
+  XS_PARSE_KEYWORD_LEXVAR,            /* padix */
+  XS_PARSE_KEYWORD_ATTRS,             /* i / {attr.name + attr.val} */
+  XS_PARSE_KEYWORD_VSTRING,           /* sv */
 
-  XS_PARSE_KEYWORD_SEQUENCE = 0x80,
-  XS_PARSE_KEYWORD_REPEATED,
-  XS_PARSE_KEYWORD_CHOICE,
-  XS_PARSE_KEYWORD_TAGGEDCHOICE,
-  XS_PARSE_KEYWORD_SEPARATEDLIST,
-  XS_PARSE_KEYWORD_PARENSCOPE = 0xb0, /* (...) */
-  XS_PARSE_KEYWORD_BRACKETSCOPE,      /* [...] */
-  XS_PARSE_KEYWORD_BRACESCOPE,        /* {...} */
-  XS_PARSE_KEYWORD_CHEVRONSCOPE,      /* <...> */
+  XS_PARSE_KEYWORD_SEQUENCE = 0x80,   /* contained */
+  XS_PARSE_KEYWORD_REPEATED,          /* i, contained */
+  XS_PARSE_KEYWORD_CHOICE,            /* i, contained */
+  XS_PARSE_KEYWORD_TAGGEDCHOICE,      /* i, contained */
+  XS_PARSE_KEYWORD_SEPARATEDLIST,     /* i, contained */
+  XS_PARSE_KEYWORD_PARENSCOPE = 0xb0, /* contained */
+  XS_PARSE_KEYWORD_BRACKETSCOPE,      /* contained */
+  XS_PARSE_KEYWORD_BRACESCOPE,        /* contained */
+  XS_PARSE_KEYWORD_CHEVRONSCOPE,      /* contained */
 };
 
 enum {
@@ -188,6 +190,7 @@ enum {
 
 #define XPK_BLOCK_flags(flags) {.type = XS_PARSE_KEYWORD_BLOCK|(flags), .u.pieces = NULL}
 #define XPK_BLOCK              XPK_BLOCK_flags(0)
+#define XPK_BLOCK_VOIDCTX      XPK_BLOCK_flags(XPK_TYPEFLAG_SPECIAL|XPK_TYPEFLAG_G_VOID)
 #define XPK_BLOCK_SCALARCTX    XPK_BLOCK_flags(XPK_TYPEFLAG_SPECIAL|XPK_TYPEFLAG_G_SCALAR)
 #define XPK_BLOCK_LISTCTX      XPK_BLOCK_flags(XPK_TYPEFLAG_SPECIAL|XPK_TYPEFLAG_G_LIST)
 
@@ -198,6 +201,7 @@ enum {
 
 #define XPK_TERMEXPR_flags(flags) {.type = XS_PARSE_KEYWORD_TERMEXPR|(flags)}
 #define XPK_TERMEXPR              XPK_TERMEXPR_flags(0)
+#define XPK_TERMEXPR_VOIDCTX      XPK_TERMEXPR_flags(XPK_TYPEFLAG_G_VOID)
 #define XPK_TERMEXPR_SCALARCTX    XPK_TERMEXPR_flags(XPK_TYPEFLAG_G_SCALAR)
 #define XPK_LISTEXPR_flags(flags) {.type = XS_PARSE_KEYWORD_LISTEXPR|(flags)}
 #define XPK_LISTEXPR              XPK_LISTEXPR_flags(0)
@@ -263,13 +267,18 @@ enum {
 #define XPK_CHEVRONSCOPE_OPT(...) \
   {.type = XS_PARSE_KEYWORD_CHEVRONSCOPE|XPK_TYPEFLAG_OPT, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
 
-typedef union {
-  OP *op;
-  CV *cv;
-  SV *sv;
-  int i;
-  struct { SV *name; SV *value; } attr;
-  PADOFFSET padix;
+typedef struct {
+  /* struct containing a single anonymous union for now
+   * this gives us room to add things like line numbers later on
+   */
+  union {
+    OP *op;
+    CV *cv;
+    SV *sv;
+    int i;
+    struct { SV *name; SV *value; } attr;
+    PADOFFSET padix;
+  };
 } XSParseKeywordPiece;
 
 struct XSParseKeywordHooks {
@@ -299,7 +308,7 @@ static void S_register_xs_parse_keyword(pTHX_ const char *kwname, const struct X
   if(!register_xs_parse_keyword_func)
     croak("Must call boot_xs_parse_keyword() first");
 
-  return (*register_xs_parse_keyword_func)(aTHX_ kwname, hooks, hookdata);
+  (*register_xs_parse_keyword_func)(aTHX_ kwname, hooks, hookdata);
 }
 
 #define boot_xs_parse_keyword(ver) S_boot_xs_parse_keyword(aTHX_ ver)
